'use client';

import { useState } from 'react';
import type { Control, FieldValues, Path } from 'react-hook-form';
import Dropzone from 'react-dropzone';
import { Cloud, File, FileCheck } from 'lucide-react';

import { startSimulatedProgress } from '~/lib/stimulatedProgress';

import { FormControl, FormField, FormItem, FormLabel, FormMessage } from '../ui/form';
import { Progress } from '../ui/progress';
import { Button } from '../ui/button';
import { Input } from '../ui/input';

type ImageInputFieldProps<TFieldValues extends FieldValues> = {
  control: Control<TFieldValues> | undefined;
  name: Path<TFieldValues>;
  label: string;
  nextStep: () => Promise<void>;
};

const ImageInputField = <TFieldValues extends FieldValues>({
  control,
  name,
  label,
  nextStep,
}: ImageInputFieldProps<TFieldValues>) => {
  const [isUploading, setIsUploading] = useState<boolean>(false);
  const [uploadProgress, setUploadProgress] = useState<number>(0);

  return (
    <FormField
      control={control}
      name={name}
      render={({ field: { value, onChange } }) => (
        <FormItem>
          <FormLabel className='text-2xl font-bold'>{label}</FormLabel>
          <FormControl>
            <Dropzone
              multiple={false}
              onDrop={(acceptedFiles) => {
                if (acceptedFiles.length > 0) {
                  onChange(acceptedFiles[0]);
                  setIsUploading(true);
                  startSimulatedProgress(setUploadProgress);
                }
              }}
            >
              {({ getRootProps, getInputProps, acceptedFiles, open }) => (
                <div
                  {...getRootProps()}
                  className='h-64 w-full rounded-lg border border-dashed border-gray-300'
                  onClick={(e) => {
                    e.stopPropagation();
                    open(); // Open file dialog when clicked
                  }}
                >
                  <div className='flex h-full w-full items-center justify-between'>
                    <label
                      htmlFor='dropzone-file'
                      className='flex h-full w-full cursor-pointer flex-col items-center justify-center rounded-lg bg-gray-50 hover:bg-gray-100'
                    >
                      <div className='flex flex-col items-center justify-center pb-6 pt-5'>
                        <Cloud className='mb-2 h-6 w-6 text-zinc-500' />
                        <p className='mb-2 text-sm text-zinc-700'>
                          <span className='font-semibold'>Carica </span>
                          un&apos;immagine
                        </p>
                      </div>
                      {acceptedFiles ? (
                        <>
                          {acceptedFiles[0] ? (
                            <div className='flex max-w-xs items-center divide-x divide-zinc-200 overflow-hidden rounded-md bg-white outline outline-[1px] outline-zinc-200'>
                              <div className='grid h-full place-items-center px-3 py-2'>
                                <File className='h-4 w-4 text-blue-500' />
                              </div>
                              <div className='h-full truncate px-3 py-2 text-sm text-black'>
                                {acceptedFiles[0].name}
                              </div>
                            </div>
                          ) : null}
                        </>
                      ) : null}
                      {isUploading && (
                        <div className='mx-auto mt-4 w-full max-w-xs'>
                          <Progress value={uploadProgress} className='h-1 w-full bg-zinc-200' />
                          {uploadProgress === 100 && (
                            <div className='flex items-center justify-center gap-1 pt-2 text-center text-sm text-zinc-700'>
                              <FileCheck className='h-3 w-3' />
                              Immagine Caricata
                            </div>
                          )}
                        </div>
                      )}
                      {/* Hidden input field for Dropzone */}
                      <Input {...getInputProps()} type='file' accept='image/*' className='hidden' />
                    </label>
                  </div>
                </div>
              )}
            </Dropzone>
          </FormControl>
          <FormMessage />
          {value ? (
            <Button
              disabled={uploadProgress < 100}
              onClick={async (e) => {
                e.preventDefault();
                await nextStep();
              }}
            >
              OK
            </Button>
          ) : null}
        </FormItem>
      )}
    />
  );
};

export default ImageInputField;
